#include "apply_exception.h"
#include "printlog.h"
#include "utilities.h"

int get_read_exception_by_dentry(int huid, struct integrity_state_struct* integrity_state, struct apply_exception_struct* aes, struct dentry* dentry)
{
	struct apply_exception_struct* ae;
	struct file_exception_struct* fe;
	char* path;

	if (integrity_state == NULL || aes == NULL || dentry == NULL) {
		print_log(ERR_INF, "null error in [get_read_exception_by_dentry].\n");
		return 0;
	}
	
	path = kmalloc(MAX_PATH_LEN, GFP_KERNEL);
	if (path == NULL) {
		print_log(ERR_INF, "null error in [ger_read_exception_by_dentry].\n");
		return 0;
	}
	
	if (get_path_from_dentry(dentry, path) != 0) {
		print_log(ERR_INF, "get path from dentry error.\n");
		kfree(path);
		return 0;
	}
	
	list_for_each_entry(ae, &aes->list, list) {
		if (ae->exception != NULL && (ae->exception->user_id == 0 || ae->exception->user_id == huid)) {
			list_for_each_entry(fe, &(ae->exception->file_exception.list), list) {
				if (fe->mode == CAN_READ || fe->mode == CAN_WRITE) {
					if (strcmp(path, fe->path) == 0) {
						kfree(path);
						return 1;
					}
				}
				else if (fe->mode == CAN_RECURSIVE_READ || fe->mode == CAN_RECURSIVE_WRITE) {
					if (strlen(path) >= strlen(fe->path) && strncmp(fe->path, path, strlen(fe->path)) == 0) {
						kfree(path);
						return 1;
					}
				}
			}
		}
	}
	kfree(path);
	return 0;
}

int get_write_exception_by_dentry(int huid, struct integrity_state_struct* integrity_state, struct apply_exception_struct* aes, struct dentry* dentry)
{
	struct apply_exception_struct* ae;
	struct file_exception_struct* fe;
	char* path;

	if (integrity_state == NULL || aes == NULL || dentry == NULL) {
		print_log(ERR_INF, "null error in [get_write_exception_by_dentry].\n");
		return 0;
	}
	
	path = kmalloc(MAX_PATH_LEN, GFP_KERNEL);
	if (path == NULL) {
		print_log(ERR_INF, "null error in [ger_read_exception_by_dentry].\n");
		return 0;
	}
	
	if (get_path_from_dentry(dentry, path) != 0) {
		print_log(ERR_INF, "get path from dentry error.\n");
		kfree(path);
		return 0;
	}
	
	list_for_each_entry(ae, &aes->list, list) {
		if (ae->exception != NULL && (ae->exception->user_id == 0 || ae->exception->user_id == huid)) {
			list_for_each_entry(fe, &ae->exception->file_exception.list, list) {
				if (fe->mode == CAN_WRITE) {
					if (strcmp(path, fe->path) == 0) {
						kfree(path);
						return 1;
					}
				}
				else if (fe->mode == CAN_RECURSIVE_WRITE) {
					if (strlen(path) >= strlen(fe->path) && strncmp(fe->path, path, strlen(fe->path)) == 0) {
						kfree(path);
						return 1;
					}
				}
			}
		}
	}
	kfree(path);
	return 0;
}

int get_exec_exception(int huid, struct integrity_state_struct* integrity_state, struct apply_exception_struct* aes, char* new_bin)
{
	struct apply_exception_struct* ae;
	struct exec_exception_struct* ee;

	if (integrity_state == NULL || aes == NULL || new_bin == NULL) {
		print_log(ERR_INF, "null error in [get_exec_exception].\n");
		return 0;
	}

	list_for_each_entry(ae, &aes->list, list) {
		if (ae->exception != NULL && (ae->exception->user_id == 0 || ae->exception->user_id == huid)) {
			list_for_each_entry(ee, &ae->exception->exec_exception.list, list) {
				if (strcmp(new_bin, ee->path) == 0) {
					return 1;
				}
			}
		}
	}
	
	return 0;
}

int get_execinh_exception(int huid, struct integrity_state_struct* integrity_state, struct apply_exception_struct* aes, char* new_bin)
{
	struct apply_exception_struct* ae;
	struct exec_exception_struct* ee;

	if (integrity_state == NULL || aes == NULL || new_bin == NULL) {
		print_log(ERR_INF, "null error in [get_execinh_exception].\n");
		return 0;
	}

	list_for_each_entry(ae, &aes->list, list) {
		if (ae->exception != NULL && (ae->exception->user_id == 0 || ae->exception->user_id == huid)) {
			list_for_each_entry(ee, &ae->exception->execinh_exception.list, list) {
				if (strcmp(new_bin, ee->path) == 0) {
					return 1;
				}
			}
		}
	}
	
	return 0;
}

int get_fpp_exception(int huid, char* bin, struct apply_exception_struct* aes)
{
	struct apply_exception_struct* ae;

	if (bin == NULL || aes == NULL) {
		print_log(ERR_INF, "null error in [get_fpp_exception].\n");
		return 0;
	}

	list_for_each_entry(ae, &aes->list, list) {
		if (ae->exception != NULL) {
			if (ae->exception->user_id == 0 && ae->exception->fpp == 1) {
				return 1;
			}
		}
	}
	
	return 0;
}

int get_drop_exception(int huid, struct integrity_state_struct* integrity_state, struct apply_exception_struct* aes, struct dentry* dentry)
{
	struct apply_exception_struct* ae;
	struct file_exception_struct* fe;
	char* path;

	if (integrity_state == NULL || aes == NULL || dentry == NULL) {
		print_log(ERR_INF, "null error in [get_drop_exception].\n");
		return 0;
	}
	
	path = kmalloc(MAX_PATH_LEN, GFP_KERNEL);
	if (path == NULL) {
		print_log(ERR_INF, "null error in [ger_read_exception_by_dentry].\n");
		return 0;
	}
	
	if (get_path_from_dentry(dentry, path) != 0) {
		print_log(ERR_INF, "get path from dentry error.\n");
		kfree(path);
		return 0;
	}
	
	list_for_each_entry(ae, &aes->list, list) {
		if (ae->exception != NULL && (ae->exception->user_id == 0 || ae->exception->user_id == huid)) {
			list_for_each_entry(fe, &ae->exception->file_exception.list, list) {
				if (fe->effect == STAY) {
					if (fe->mode == CAN_WRITE) {
						if (strcmp(path, fe->path) == 0) {
							kfree(path);
							return 1;
						}
					}
					else if (fe->mode == CAN_RECURSIVE_WRITE) {
						if (strlen(path) >= strlen(fe->path) && strncmp(fe->path, path, strlen(fe->path)) == 0) {
							kfree(path);
							return 1;
						}
					}
				}
			}
		}
	}
	kfree(path);
	return 0;
}

int get_cap_exception(int huid, struct integrity_state_struct* integrity_state, struct apply_exception_struct* aes, int cap)
{
	struct apply_exception_struct* ae;

	if (integrity_state == NULL || aes == NULL || cap < 0 || cap >= 32) {
		print_log(ERR_INF, "null error in [get_drop_exception].\n");
		return 0;
	}
	
	list_for_each_entry(ae, &aes->list, list) {
		if (ae->exception != NULL && (ae->exception->user_id == 0 || ae->exception->user_id == huid)) {
			if ((ae->exception->cap & bit_power[cap]) > 0) {
				return 1;
			}
		}
	}
	return 0;
}

int get_rap_exception(int huid, struct apply_exception_struct* aes)
{
	struct apply_exception_struct* ae;

	if (aes == NULL) {
		print_log(ERR_INF, "null error in [get_rap_exception].\n");
		return 0;
	}
	
	list_for_each_entry(ae, &aes->list, list) {
		if (ae->exception != NULL) {
			if (ae->exception->user_id == 0 && ae->exception->rap == 1) {
				return 1;
			}
		}
	}
	return 0;	
}

int get_lsp_exception(int huid, struct apply_exception_struct* aes)
{
	struct apply_exception_struct* ae;

	if (aes == NULL) {
		print_log(ERR_INF, "null error in [get_lsp_exception].\n");
		return 0;
	}
	
	list_for_each_entry(ae, &aes->list, list) {
		if (ae->exception != NULL) {
			if (ae->exception->user_id == 0 && ae->exception->lsp == 1) {
				return 1;
			}
		}
	}
	return 0;	
}
