#include "filesystem.h"
#include "printlog.h"
#include "readpolicy.h"

struct namespace* init_namespace;
struct fs_prot_struct* fs_prot_list;

int get_fs_prot_level(char* dev_name)
{
	struct fs_prot_struct* cur;
	
	if (dev_name == NULL) {
		print_log(ERR_INF, "null error in [get_fs_prot_level]\n");
		return FS_NO_MOUNT_POINT;
	}

	//print_log(ERR_INF, dev_name);
	list_for_each_entry(cur, &fs_prot_list->list, list) {
		if (strcmp(cur->dev_name, dev_name) == 0) {
			return cur->prot_level;
		}
	}
	return FS_UNPROTECTED;
}

int is_special_fs_type(char* type)
{
	if (type == NULL) {
		print_log(ERR_INF, "null error in [is_special_fs_type].\n");
		return 0;
	}
	if (strcmp(type, "devpts") == 0 || strcmp(type, "proc") == 0 || strcmp(type, "sysfs") == 0) {
		return 1;
	}
	
	if (strcmp(type, "bdev") == 0 || strcmp(type, "eventpollfs") == 0 || strcmp(type, "futexfs") == 0) {
		return 1;
	}

	if (strcmp(type, "pipefs") == 0 || strcmp(type, "rootfs") == 0 || strcmp(type, "shm") == 0) {
		return 1;
	}

	if (strcmp(type, "sockfs") == 0 || strcmp(type, "ramfs") == 0) {
		return 1;
	}

	if (strcmp(type, "usbfs") == 0 || strcmp(type, "mqueue") == 0 || strcmp(type, "binfmt_misc") == 0 || strcmp(type, "tmpfs") == 0) {
		return 1;
	}

	return 0;
}

int check_special_fs_type(char* type, char* path)
{
	if (strcmp(type, "devpts") == 0) {
		if (strcmp(path, "/dev/pts") == 0) {
			return 1;
		}
		else {
			return 0;
		}
	}
	else if (strcmp(type, "proc") == 0) {
		if (strcmp(path, "/proc") == 0) {
			return 1;
		}
		else {
			return 0;
		}
	}
	else if (strcmp(type, "sysfs") == 0) {
		if (strcmp(path, "/sys") == 0) {
			return 1;
		}
		else {
			return 0;
		}
	}
	return 1;
}

void release_fs_prot_list(struct fs_prot_struct* fss)
{
}

void print_fs_policy(struct fs_prot_struct* fss)
{
	struct fs_prot_struct* cur;
	char buf[256];

	list_for_each_entry(cur, &fss->list, list) {
		sprintf(buf, "%s %s %d\n", cur->dev_name, cur->mount_point, cur->prot_level);
		print_log(SIM_INF, buf);
	}	
}

int read_single_fs_prot_policy(struct fs_prot_struct** fss)
{
	struct file* f;	
	struct fs_prot_struct* fs_prot;
	int p;
	char buf[256];
	
	*fss = kmalloc(sizeof(struct fs_prot_struct), GFP_KERNEL);
	if (*fss == NULL) {
		print_log(ERR_INF, "nomem error in [read_fs_prot_policy]. \n");
		return -1;
	}	
	INIT_LIST_HEAD(&(*fss)->list);
	
	f = open_file(FS_POLICY_PATH);
	if (f == NULL) {
		print_log(ERR_INF, "open file error in [read_fs_prot_policy].\n");
		return -1;
	}
	
	do {
		p = get_str(f, buf, ' ');
		if (p == -1) {
			break;
		}
		
		fs_prot = kmalloc(sizeof(struct fs_prot_struct), GFP_KERNEL);
		if (fs_prot == NULL) {
			print_log(ERR_INF, "nomem error in [read_fs_prot_policy].\n");
			release_fs_prot_list(*fss);
			return -1;
		}
		fs_prot->dev_name = NULL;
		fs_prot->mount_point = NULL;
		fs_prot->prot_level = 0;
		list_add_tail(&(fs_prot->list), &(*fss)->list);
		
		fs_prot->dev_name = kmalloc(strlen(buf) + 1, GFP_KERNEL);
		if (fs_prot == NULL) {
			print_log(ERR_INF, "nomem error in [read_fs_prot_policy].\n");
			release_fs_prot_list(*fss);
			return -1;
		}
		strcpy(fs_prot->dev_name, buf);

		p = alloc_str(f, &fs_prot->mount_point, ' ');
		if (p == -1) {
			print_log(ERR_INF, "read fs policy error: mountpoint.\n");
			release_fs_prot_list(*fss);
			return -1;
		}
		
		p = get_str(f, buf, '\n');
		if (strcmp(buf, "TRUSTED") == 0) {
			fs_prot->prot_level = FS_TRUSTED;
		}
		else if (strcmp(buf, "PROTECTED") == 0) {
			fs_prot->prot_level = FS_PROTECTED;
		}
		else {
			print_log(ERR_INF, "read fs policy error: prot_level.\n");
			release_fs_prot_list(*fss);
			return -1;			
		}
	} while (1);
	print_fs_policy(*fss);
	return 0;
}

int read_fs_prot_policy(void)
{
	return read_single_fs_prot_policy(&fs_prot_list);
}
