#include "hipp.h"
#include "printlog.h"
#include "readpolicy.h"
#include "hooks.h"
#include "utilities.h"
#include "filesystem.h"
#include "pstate.h"

#define NETLINK_MASK	32323232
#define NETLINK_UMIP 	22
#define MAX_PAYLOAD 	1024

static struct sock *nl_sk = NULL;

unsigned int dev_ino;
unsigned int tty_ino;

int error_code;

static char buf[64];

static void nl_data_ready(struct sock *sk, int len)
{
	struct sk_buff *skb = NULL;
	struct nlmsghdr *nlh = NULL;
	struct task_struct* tsk = NULL;
	struct pstate_struct* pstate = NULL;
	u32 pid;     
	u8* p = NULL;
	int a, b;
	
	while ((skb = skb_dequeue(&sk->sk_receive_queue)) != NULL) {
		nlh = (struct nlmsghdr *)skb->data;
		p = NLMSG_DATA(nlh);
	
		a = *((int*)(p));
		b = *((int*)(p + 4));
		sprintf(buf, "netlink received: %d %d \n", a, b);
		print_log(WAR_INF, buf);
		
		if (a == 1) {
			if (b == 0) {
				error_code = 0;
				sprintf(buf, "change to auditing mode.");
			}
			else if (b == 1) {
				error_code = -1;
				sprintf(buf, "change to enforcing mode.");
			}
		}
		else if (a == 2) {
			tsk = find_task_by_pid(b);
			pstate = (struct pstate_struct*)get_pstate(tsk);
			if (pstate != NULL) {
				if (pstate->integrity_state.integrity_level == HIGH_INTEGRITY) {
					sprintf(buf, "pid %d : high.", b);
				}
				else {
					sprintf(buf, "pid %d : low.", b);
				}
			}
			else {
				sprintf(buf, "pid %d : error.", b);
			}
		}
		else if (a == 3) {			
			tsk = find_task_by_pid(b);
			pstate = (struct pstate_struct*)get_pstate(tsk);
			if (pstate != NULL) {
				pstate->integrity_state.integrity_level = LOW_INTEGRITY;
				sprintf(buf, "pid %d : drop.", b);
			}
			else {
				sprintf(buf, "pid %d : error.", b);
			}
		}

		strcpy(NLMSG_DATA(nlh), buf);

		pid = nlh->nlmsg_pid;
		//NETLINK_CB(skb).groups = 0;
		NETLINK_CB(skb).pid = 0;
		NETLINK_CB(skb).dst_pid = pid;
		NETLINK_CB(skb).dst_group = 0; 
		netlink_unicast(nl_sk, skb, pid, MSG_DONTWAIT);
	}
}

static int __init hipp_init(void)
{
	int i;
	int p;

	struct dentry* dentry;
	dentry = get_dentry_from_path("/dev/null");
	dev_ino = dentry->d_inode->i_ino;
	dentry = get_dentry_from_path("/dev/tty");
	tty_ino = dentry->d_inode->i_ino;
	
	init_namespace = current->namespace;
	
	error_code = 0;	
	read_magic();
	bit_power[0] = 1;
	for (i=1; i<32; i++) {
		bit_power[i] = bit_power[i - 1] * 2;
	}
	
	p = read_whole();
	if (p == -1) {
		return -1;
	}
	
	p = read_fs_prot_policy();
	if (p == -1) {
		return -1;
	}
	
	nl_sk = netlink_kernel_create(NETLINK_UMIP, 0, nl_data_ready, THIS_MODULE);

	print_log(PGS_INF, "Initialization done.\n");
	set_security_operations();
	unregister_security(security_ops);
	p = register_security(&hipp_ops);
	if (p != 0) {
		print_log(ERR_INF, "Register security operations failed.\n");
		return -1;
	}
	print_log(PGS_INF, "Security operations registered.\n");
	return 0;
}

static void __exit hipp_exit(void)
{
	sock_release(nl_sk->sk_socket);
	write_magic();
	unregister_security(&hipp_ops);
	print_log(PGS_INF, "Module unloaded.\n");
}

module_init(hipp_init);
module_exit(hipp_exit);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("A module to implement HIPP");
