#include "printlog.h"
#include "hipp.h"
#include "pstate.h"
#include "utilities.h"

static int magic;

void print_log(int type, const char* str)
{
	char* final_buf;
	
	magic = HIPP_MAGIC % 10000;
	
	final_buf = kmalloc(1024, GFP_KERNEL);
	if (final_buf == NULL) {
		return;
	}
	/*	
	if (type == WAR_INF) {
		sprintf(final_buf, "SECURITY_MODULE_HIPP%d Warning %s", magic, str);
		printk(final_buf);
	}
	kfree(final_buf);
	return;
	*/
	if (type == ERR_INF) {
		sprintf(final_buf, "SECURITY_MODULE_HIPP%d Error %s", magic, str);
	}
	else if (type == PGS_INF) {
		sprintf(final_buf, "SECURITY_MODULE_HIPP%d Progress %s", magic, str);
	}
	else if (type == WAR_INF) {
		sprintf(final_buf, "SECURITY_MODULE_HIPP%d Warning %s", magic, str);
	}
	else if (type == SIM_INF) {
		sprintf(final_buf, "SECURITY_MODULE_HIPP%d Simulation %s", magic, str);
	}
	else if (type == CTN_INF) {
		sprintf(final_buf, "%s", str);
		printk("%s", final_buf);
		kfree(final_buf);
		return;
	}
	else if (type == SPE_INF) {
		sprintf(final_buf, "SECURITY_MODULE_HIPP%d SPECIAL %s", magic, str);		
	}
	else if (type == EXP_INF) {
		sprintf(final_buf, "SECURITY_MODULE_HIPP%d Exception %s", magic, str);		
	}
	printk(KERN_INFO "%s", final_buf);
	kfree(final_buf);
}

void print_prs(int type, struct task_struct* tsk)
{
	char buf[300];
	struct pstate_struct* pstate;
	
	if (tsk == NULL) {
		return;
	}
	pstate = (struct pstate_struct*)get_pstate(tsk);
	if (pstate == NULL) {
		return;
	}
	sprintf(buf, "[task(%s) pid(%d) uid(%d) euid(%d) suid(%d) label(%d) huid(%d)] (%s)", 
		tsk->comm, tsk->pid, tsk->uid, tsk->euid, tsk->suid, pstate->integrity_state.integrity_level, pstate->huid, pstate->path);
	print_log(type, buf);
}

void format_mode(unsigned short int m, char* file_mode)
{
	char temp[20];
	int n;
	
	strcpy(temp, "rwxrwxrwx");
	n = strlen(temp);
	file_mode[n] = '\0';
	while (n > 0) 
	{
		n--;
		if ((m & 1) > 0)
		{
			file_mode[n] = temp[n];
		}
		else
		{
			file_mode[n] = '-';
		}
		m = (m >> 1);
	}
}

void get_fstype(struct inode* inode, char* fstype)
{
	fstype[0] = '\0';
	if (inode != NULL && inode->i_sb != NULL && inode->i_sb->s_type != NULL && inode->i_sb->s_type->name != NULL)
	{
		strcpy(fstype, inode->i_sb->s_type->name);
	}
}

int get_ftype(struct inode* inode)
{
	if (inode == NULL) {
		return -1;
	}
	if (S_ISLNK(inode->i_mode)) {
		return 1;
	}
	else if (S_ISREG(inode->i_mode)) {
		return 2;
	}
	else if (S_ISDIR(inode->i_mode)) {
		return 3;
	}
	else if (S_ISCHR(inode->i_mode)) {
		return 4;
	}
	else if (S_ISBLK(inode->i_mode)) {
		return 5;
	}
	else if (S_ISFIFO(inode->i_mode)) {
		return 6;
	}
	else if (S_ISSOCK(inode->i_mode)) {
		return 7;
	}
	return 8;
}

void print_den(int type, struct dentry* dentry)
{
	int p, q;
	char* path;
	char* buf;
	char mode[32];
	char fstype[32];
	
	if (dentry == NULL) {
		return;
	}

	path = kmalloc(MAX_PATH_LEN, GFP_KERNEL);
	if (path == NULL) {
		print_log(ERR_INF, "null error in [print_den].\n");
		return;
	}
	
	if (get_path_from_dentry(dentry, path) == -1) {
		kfree(path);
		return;
	}
	if (dentry->d_inode != NULL) {
		p = dentry->d_inode->i_uid;
		format_mode(dentry->d_inode->i_mode, mode);
	}
	else {
		p = -1;
		mode[0] = '\0';
	}
	get_fstype(dentry->d_inode, fstype);
	q = get_ftype(dentry->d_inode);
	
	buf = kmalloc(1024, GFP_KERNEL);
	if (buf == NULL) {
		print_log(ERR_INF, "null error in [print_den].\n");
		kfree(path);
		return;
	}
	
	sprintf(buf, "%s [%d][%s][%s][%d]", path, p, mode, fstype, q);
	print_log(type, buf);
	kfree(path);
	kfree(buf);
}
