#ifndef PSTATE_H
#define PSTATE_H

#include "hipp.h"

/* applied exception list */
struct apply_exception_struct {
	struct exception_struct* exception;
	struct list_head list;	
};

struct integrity_state_struct {
	unsigned char integrity_level;
	int mid_user_id;
};

struct shm_state_struct;

struct shm_state_list_struct {
	struct shm_state_struct* shm_state;
	struct list_head list;
};

/* process state */
struct pstate_struct {
	int magic;
	struct task_struct* tsk;
	// the path of the executable
	char* path;			
	// integrity state
	struct integrity_state_struct integrity_state;
	// current integrity level of the process
	struct apply_exception_struct apply_exceptions;
	// states of the share memory segments attached
	int huid;
	struct shm_state_list_struct shm_state_list;
};

struct istate_struct {
	int magic;
	int integrity_level;
	int mid_user_id;
};

struct pstate_list_struct {
	struct pstate_struct* pstate;
	struct list_head list;
};

struct shm_state_struct {
	int magic;
	int integrity_level;
	int mid_user_id;
	struct pstate_list_struct pstate_list;
};

int is_valid_pstate(struct pstate_struct* pstate);
int is_valid_istate(struct istate_struct* istate);
void release_apply_exceptions(struct apply_exception_struct* es);
int copy_apply_exceptions(struct apply_exception_struct* a2, struct apply_exception_struct* a1);
int establish_exception(struct apply_exception_struct* aes, char* new_bin);
int initialize_integrity_state(struct integrity_state_struct* i1, struct integrity_state_struct* i2);
struct pstate_struct* alloc_pstate(struct task_struct* tsk, struct task_struct* parent_tsk);
void free_pstate(struct task_struct* tsk);
struct istate_struct* alloc_istate(struct inode* inode);
struct pstate_struct* get_pstate(struct task_struct* tsk);
struct pstate_struct* current_pstate(void);
int exec_program(struct pstate_struct* pstate, char* new_bin, struct dentry* dentry);
struct istate_struct* get_istate(struct inode* inode);
struct istate_struct* get_mstate(struct msg_msg* msg);
struct istate_struct* alloc_mstate(struct msg_msg* msg);
int get_huid(struct task_struct* tsk);
int recv_net(struct pstate_struct* pstate);
int print_apply_exception(struct apply_exception_struct* aes);
int update_huid(int a, int b);
struct shm_state_struct* get_shm_state(struct shmid_kernel* shp);
struct shm_state_struct* alloc_shm_state(struct shmid_kernel* shp);
int is_valid_shm_state(struct shm_state_struct* shm_state);
struct istate_struct* alloc_sstate(struct sock* sk);
struct istate_struct* get_sstate(struct sock* sk);

#endif
