#include "readpolicy.h"
#include "hipp.h"
#include "printlog.h"
#include "utilities.h"
#include "capalist.h"

#define MAGIC_FILE_NAME	"/etc/umip/magic.txt"
#define	MAGIC_LEN	10

struct exception_struct* global_exception = NULL;
struct account_list_struct* account_list = NULL;
unsigned int bit_power[32];

int HIPP_MAGIC;

struct file* open_file(char* fname)
{
	mm_segment_t orig_fs;
	struct file* f = NULL;

	f = filp_open(fname, O_RDONLY, 00);
	if (IS_ERR(f)) {
		return NULL;
	}
	f->f_pos = 0;
	orig_fs = get_fs();
	set_fs(KERNEL_DS);
	return f;
}

void close_file(struct file* f)
{
	filp_close(f, NULL);
}

int get_int(struct file* f, int* res)
{
	char c;
	int p;
	int k = -1;
	
	do {
		p = f->f_op->read(f, &c, 1, &f->f_pos);
		
		if (p != 1) {
			return -1;
		}
		
		if (c >= '0' && c <= '9') {
			break;
		}
	} while (1);
	
	while (c >= '0' && c <= '9') {
		if (k == -1) {
			k = 0;
		}
		k = k * 10 + (c - '0');
		if (k >= 100000000) {
			k = -1;
			return -1;
		}
		p = f->f_op->read(f, &c, 1, &f->f_pos);

		if (p != 1) {
			break;
		}
	}

	*res = k;
	if (k == -1) {
		return -1;
	}
	return 0;
}

int get_str(struct file* f, char* buf, char div)
{
	char c;
	int p;
	int k = 0;
	
	buf[0] = '\0';
	do {
		p = f->f_op->read(f, &c, 1, &f->f_pos);
		
		if (p != 1) {
			buf[0] = '\0';
			return -1;
		}
		
		if (c == div) {
			buf[0] = '\0';
			return -1;
		}
		
		if (c == ' ' || c == '\r' || c == '\n' || c == '\t') {
			
		} else {
			break;
		}
	} while (1);
	
	while (c != '\r' && c != '\n' && c != '\t' && c != div) {
		buf[k] = c;
		k++;
		
		if (k >= 255) {
			return -1;
		}
		p = f->f_op->read(f, &c, 1, &f->f_pos);

		if (p != 1) {
			break;
		}
	}
	buf[k] = '\0';
	if (strlen(buf) == 0) {
		return -1;
	}
	return 0;
}

int alloc_str(struct file* f, char** r, char div)
{
	int p;
	char buf[256];
	
	p = get_str(f, buf, div);
	if (p == -1) {
		*r = NULL;
		return -1;
	}

	*r = kmalloc(strlen(buf) + 1, GFP_KERNEL);
	if (*r == NULL) {
		print_log(ERR_INF, "nomem error in [alloc_str]. \n");
		return -1;
	}
	strcpy(*r, buf);
	return 0;
}

void release_account_list(struct account_list_struct* acs)
{
}

int read_passwd_file(struct account_list_struct** acs)
{
	struct file* f;
	struct account_list_struct* ac;
	char buf[256];
	int p;
	
	*acs = kmalloc(sizeof(struct account_list_struct), GFP_KERNEL);
	if (*acs == NULL) {
		print_log(ERR_INF, "nomem error in [read_passwd_file]. \n");
		return -1;
	}	
	INIT_LIST_HEAD(&(*acs)->list);
	
	f = open_file("/etc/passwd");
	if (f == NULL) {
		print_log(ERR_INF, "the file /etc/passwd does not exist.\n");
		release_account_list(*acs);
		return -1;
	}
	
	do {
		ac = kmalloc(sizeof(struct account_list_struct), GFP_KERNEL);
		if (ac == NULL) {
			print_log(ERR_INF, "nomem error in [read_passwd_file].\n");
			return -1;
		}

		p = alloc_str(f, &(ac->account_name), ':');
		if (p == -1) {
			kfree(ac);
			break;
		}

		p = get_int(f, &(ac->account_number));
		if (ac->account_number == -1) {
			print_log(ERR_INF, "read account number error.\n");
			kfree(ac->account_name);
			kfree(ac);
			release_account_list(*acs);
			close_file(f);
			return -1;
		}
		
		if (ac->account_number < 500 || ac->account_number >= 10000) {
			kfree(ac->account_name);
			kfree(ac);
			p = get_str(f, buf, '\n');
			if (p == -1) {
				print_log(ERR_INF, "read through line error.\n");
				kfree(ac->account_name);
				kfree(ac);
				release_account_list(*acs);
				close_file(f);
				return -1;
			}
			continue;
		}
		
		ac->home_directory = NULL;
		ac->user_exception = NULL;
		list_add_tail(&(ac->list), &(*acs)->list);
		
		p = get_int(f, &(ac->group_number));
		if (ac->group_number == -1) {
			print_log(ERR_INF, "read group number error.\n");
			close_file(f);
			release_account_list(*acs);
			return -1;
		}
		
		p = get_str(f, buf, ':');
		if (p == -1) {
			print_log(ERR_INF, "read user full name error.\n");
			close_file(f);
			release_account_list(*acs);
			return -1;
		}
		
		p = alloc_str(f, &ac->home_directory, ':');
		if (p == -1) {
			print_log(ERR_INF, "read home directory error.\n");
			close_file(f);
			release_account_list(*acs);
			return -1;
		}

		p = get_str(f, buf, '\n');
		if (p == -1) {
			print_log(ERR_INF, "read user shell error.\n");
			close_file(f);
			release_account_list(*acs);
			return -1;
		}
		ac->user_type = 0;
	} while (1);
	close_file(f);
	return 0;
}

void release_exceptions(struct exception_struct* es)
{
}

int read_policy_file(int user_id, char* path, struct exception_struct** es)
{
	struct file* f;
	struct exception_struct* e = NULL;
	struct file_exception_struct* fe = NULL;
	struct exec_exception_struct* ee = NULL;
	int k;
	int p;
	char buf[256];
	int b_rap = 0, b_fpp = 0, b_lsp = 0;
	
	*es = kmalloc(sizeof(struct exception_struct), GFP_KERNEL);
	if (*es == NULL) {
		print_log(ERR_INF, "nomem error in [read_policy_file]. \n");
		return -1;
	}	
	INIT_LIST_HEAD(&(*es)->list);

	f = open_file(path);
	if (f == NULL) {
		sprintf(buf, "the user policy file %s does not exists.\n", path);
		print_log(PGS_INF, buf);
		return 0;
	}
	
	do {
		p = get_str(f, buf, ' ');
		if (p == -1) {
			break;
		}
		
		if (strcmp(buf, "ProgramPath") == 0) {
			e = kmalloc(sizeof(struct exception_struct), GFP_KERNEL);
			if (e == NULL) {
				print_log(ERR_INF, "nomem error in [read_policy_file].\n");
				return -1;
			}

			e->user_id = user_id;
			e->path = NULL;
			e->rap = 0;
			e->fpp = 0;
			e->lsp = 0;
			e->cap = DEFAULT_CAPABILITY;
			INIT_LIST_HEAD(&(e->file_exception.list));
			INIT_LIST_HEAD(&(e->exec_exception.list));
			INIT_LIST_HEAD(&(e->execinh_exception.list));
			list_add_tail(&(e->list), &(*es)->list);
			
			p = alloc_str(f, &(e->path), ' ');
			if (p == -1) {
				print_log(ERR_INF, "read program path error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}

			b_rap = 0;
			b_fpp = 0;
			b_lsp = 0;
		} else if (strcmp(buf, "RemoteAdmPoint") == 0) {
			if (e == NULL) {
				print_log(ERR_INF, "missing program path in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1;
			}
			if (b_rap == 1) {
				print_log(ERR_INF, "multiple RemoteAdmPoint in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1;
			}
			b_rap = 1;

			p = get_str(f, buf, ' ');
			if (p == -1) {
				print_log(ERR_INF, "read RemoteAdmPoint error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}
			if (strcmp(buf, "yes") == 0) {
				e->rap = 1;
			}
			else if (strcmp(buf, "no") == 0) {
				e->rap = 0;
			}
			else {
				print_log(ERR_INF, "RemoteAdmPoint error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}
		} else if (strcmp(buf, "LocalServicePoint") == 0) {
			if (e == NULL) {
				print_log(ERR_INF, "missing program path in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1;
			}
			if (b_lsp == 1) {
				print_log(ERR_INF, "multiple LocalServicePoint in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1;
			}
			b_lsp = 1;

			p = get_str(f, buf, ' ');
			if (p == -1) {
				print_log(ERR_INF, "read LocalServicePoint error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}
			if (strcmp(buf, "yes") == 0) {
				e->lsp = 1;
			}
			else if (strcmp(buf, "no") == 0) {
				e->lsp = 0;
			}
			else {
				print_log(ERR_INF, "LocalServicePoint error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}
		} else if (strcmp(buf, "FileProcessPoint") == 0) {
			if (e == NULL) {
				print_log(ERR_INF, "missing program path in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1;
			}
			if (b_fpp == 1) {
				print_log(ERR_INF, "multiple FileProcessPoint in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1;
			}
			b_fpp = 1;

			p = get_str(f, buf, ' ');
			if (p == -1) {
				print_log(ERR_INF, "read FileProcessPoint error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}
			if (strcmp(buf, "yes") == 0) {
				e->fpp = 1;
			}
			else if (strcmp(buf, "no") == 0) {
				e->fpp = 0;
			}
			else {
				print_log(ERR_INF, "FileProcessPoint error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}
		} else if (strcmp(buf, "FileException") == 0) {
			if (e == NULL) {
				print_log(ERR_INF, "missing program path in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1;
			}
			fe = kmalloc(sizeof(struct file_exception_struct), GFP_KERNEL);
			if (fe == NULL) {
				print_log(ERR_INF, "nomem error in [read_policy_file].\n");
				return -1;
			}
			fe->path = NULL;
			list_add_tail(&(fe->list), &(e->file_exception.list));
			
			p = alloc_str(f, &(fe->path), ' ');
			if (p == -1) {
				print_log(ERR_INF, "read file path error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}
			p = get_str(f, buf, ' ');
			if (p == -1) {
				print_log(ERR_INF, "read file mode error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}
			if (strcmp(buf, "r") == 0) {
				fe->mode = CAN_READ;
			}
			else if (strcmp(buf, "r_recursive") == 0) {
				fe->mode = CAN_RECURSIVE_READ;
			}
			else if (strcmp(buf, "w") == 0) {
				fe->mode = CAN_WRITE;
			}
			else if (strcmp(buf, "w_recursive") == 0) {
				fe->mode = CAN_RECURSIVE_WRITE;
			}
			else {
				print_log(ERR_INF, "file mode error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}
			
			fe->effect = 0;
			p = get_str(f, buf, '\n');
			if (p == -1) {
				/*
				print_log(ERR_INF, "read file effect error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
				*/
				fe->effect= DROP;
				continue;
			}
			if (strcmp(buf, "drop") == 0) {
				fe->effect= DROP;
			}
			else if (strcmp(buf, "stay") == 0) {
				fe->effect= STAY;
			}
			else {
				print_log(ERR_INF, "file effect error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}
		} else if (strcmp(buf, "CapException") == 0) {
			if (e == NULL) {
				print_log(ERR_INF, "missing program path in [read_policy_file].\n");
				release_exceptions(*es);
				close_file(f);
				return -1;
			}
			/*
			k = -1;
			do {
				p = f->f_op->read(f, &c, 1, &f->f_pos);
				if (p != 1 || c == '\n' || c == '\t' || c == ' ' || c == '\r') {
					if (k != -1) {
						if (k < 0 || k >= 32) {
							print_log(ERR_INF, "read cap exception error in [read_policy_file].\n");
							close_file(f);
							release_exceptions(*es);
							return -1;
						}
						e->cap = (e->cap | bit_power[k]); 
						k = -1;
					}
				} else if (c >= '0' && c <= '9') {
					if (k == -1) {
						k = 0;
					}
					k = k * 10 + c - '0';
				} else {
					print_log(ERR_INF, "read cap exception error in [read_policy_file].\n");
					close_file(f);
					release_exceptions(*es);
					return -1;
				}
				if (c == '\n') {
					break;
				}
				if (p != 1) {
					close_file(f);
					return 0;
				}
			} while (1);
			*/
			p = get_str(f, buf, ' ');
			if (p == -1) {
				print_log(ERR_INF, "read CapException error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}
			p = 0;
			for (k=0; k<32; k++) {
				if (strcmp(cap_names[k], buf) == 0) {
					e->cap = (e->cap | bit_power[k]); 
					p = 1;					
				}
			}
			if (p == 0) {
				print_log(ERR_INF, "CapException error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}	
		} else if (strcmp(buf, "ExecException") == 0) {
			if (e == NULL) {
				print_log(ERR_INF, "missing program path in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1;
			}
			ee = kmalloc(sizeof(struct exec_exception_struct), GFP_KERNEL);
			if (ee == NULL) {
				print_log(ERR_INF, "nomem error in [read_policy_file].\n");
				return -1;
			}
			list_add_tail(&(ee->list), &(e->exec_exception.list));
			
			p = alloc_str(f, &(ee->path), ' ');
			if (p == -1) {
				print_log(ERR_INF, "read exec path error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}
		} else if (strcmp(buf, "ExecInhException") == 0) {
			if (e == NULL) {
				print_log(ERR_INF, "missing program path in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1;
			}
			ee = kmalloc(sizeof(struct exec_exception_struct), GFP_KERNEL);
			if (ee == NULL) {
				print_log(ERR_INF, "nomem error in [read_policy_file].\n");
				return -1;
			}
			list_add_tail(&(ee->list), &(e->execinh_exception.list));
			
			p = alloc_str(f, &(ee->path), ' ');
			if (p == -1) {
				print_log(ERR_INF, "read exec path error in [read_policy_file].\n");
				close_file(f);
				release_exceptions(*es);
				return -1; 
			}			
		}
		else {
			print_log(ERR_INF, "Unknown tag in [read_policy_file].\n");
			close_file(f);
			release_exceptions(*es);
			return -1;
		}
	} while (1);
	close_file(f);
	return 0;
}

void print_user(struct account_list_struct* acc)
{
	char buf[256];
	
	if (acc == NULL) {
		return;
	}
	list_for_each_entry(acc, &account_list->list, list) {
		sprintf(buf, "%s %s %d %d %d\n", acc->account_name, acc->home_directory, acc->account_number, acc->group_number, acc->user_type);
		print_log(SIM_INF, buf);
	}	
}

void print_policy(struct exception_struct* es)
{
	struct exception_struct* e;	
	struct file_exception_struct* fe;
	struct exec_exception_struct* ee;
	char buf[256];
	
	if (es == NULL) {
		return;
	}
	list_for_each_entry(e, &es->list, list) {
		sprintf(buf, "%d %s %d %d %d %u\n", e->user_id, e->path, e->rap, e->fpp, e->lsp, e->cap);
		print_log(SIM_INF, buf);
		list_for_each_entry(fe, &e->file_exception.list, list) {
			sprintf(buf, "file: %s %d %d\n", fe->path, fe->mode, fe->effect);
			print_log(SIM_INF, buf);	
		}
		list_for_each_entry(ee, &e->exec_exception.list, list) {
			sprintf(buf, "exec: %s\n", ee->path);
			print_log(SIM_INF, buf);	
		}
		print_log(SIM_INF, "#\n");	
	}	
}

int read_single_policy(int user_id, char* policy_file_path, struct exception_struct** final_es)
{
	int p;
	struct exception_struct* tmp; 
	struct exception_struct* es; 
	char buf[256];
	
	if (policy_file_path == NULL || final_es == NULL) {
		print_log(ERR_INF, "null error in [read_single_policy].\n");
		return -1;
	}
	
	p = read_policy_file(user_id, policy_file_path, &es);
	if (p == -1) {
		sprintf(buf, "read user policy file \"%s\" failed.\n", policy_file_path);
		print_log(ERR_INF, buf);
		return -1;
	}
	
	tmp = *final_es;
	*final_es = es;
	if (tmp != NULL) {
		release_exceptions(tmp);
	}
	return 0;
}

int read_some_policy(int user_id)
{
	int p;
	struct account_list_struct* ac;
	char path[128];
	
	if (user_id == 0) {
		p = read_single_policy(0, GLOBAL_POLICY_FILE_PATH, &global_exception);
		if (p == -1) {
			return -1;
		}
		print_policy(global_exception);
		return 0;
	}
	else {
		if (account_list == NULL) {
			print_log(ERR_INF, "The list of normal accounts is empty.\n");
			return -1;		
		}
		
		list_for_each_entry(ac, &account_list->list, list) {
			if (ac->account_number == user_id) {
				sprintf(path, "%s/%s", ac->home_directory, USER_POLICY_FILE_NAME);
				p = read_single_policy(user_id, path, &(ac->user_exception));
				if (p == -1) {
					return -1;
				}
				print_policy(ac->user_exception);
				return 0;
			}
		}
	}
	return -1;
}

int read_all_policy(void)
{
	int p;
	struct account_list_struct* ac;
	char path[256];
	
	p = read_single_policy(0, GLOBAL_POLICY_FILE_PATH, &global_exception);
	if (p == -1) {
		return -1;
	}
	print_policy(global_exception);
	
	if (account_list == NULL) {
		print_log(ERR_INF, "The list of normal accounts is empty.\n");
		return -1;		
	}
	
	list_for_each_entry(ac, &account_list->list, list) {
		sprintf(path, "%s/%s", ac->home_directory, USER_POLICY_FILE_NAME);
		p = read_single_policy(ac->account_number, path, &(ac->user_exception));
		if (p == -1) {
			return -1;
		}
		print_policy(ac->user_exception);
	}
	return 0;
}

int read_whole(void)
{
	int p;
	struct account_list_struct* acs;
	
	p = read_passwd_file(&acs);
	if (p == -1) {
		print_log(ERR_INF, "read the file /etc/passwd failed.\n");
		return -1;
	}
	account_list = acs;
	print_user(account_list);
	
	p = read_all_policy();
	if (p == -1) {
		return -1;
	}
	return 0;
}	

int read_magic(void)
{
	mm_segment_t orig_fs;
	int p;
	struct file* f = NULL;
	char buf[64];

	f = filp_open(MAGIC_FILE_NAME, O_RDONLY, 00);
	if (IS_ERR(f)) {
		print_log(ERR_INF, "read magic failed.\n");
		return -1;
	}
	f->f_pos = 0;
	orig_fs = get_fs();
	set_fs(KERNEL_DS);
	
	p = get_int(f, &HIPP_MAGIC);
	HIPP_MAGIC++;
	if (p == -1) {
		return -1;
	}
	filp_close(f, NULL);
	sprintf(buf, "read magic %d\n", HIPP_MAGIC);
	print_log(PGS_INF, buf);
	return 0;
}

int write_magic(void)
{
	mm_segment_t orig_fs;
	struct file* f = NULL;
	char buf[32];
	int i, tmp, p;

	f = filp_open(MAGIC_FILE_NAME, O_WRONLY | O_CREAT, 0600);

	if (IS_ERR(f)) {
		print_log(ERR_INF, "write magic failed.\n");
		return -1;
	}
	f->f_pos = 0;
	orig_fs = get_fs();
	set_fs(KERNEL_DS);	

	buf[MAGIC_LEN] = '\0';
	tmp = HIPP_MAGIC;
	for (i=0; i<MAGIC_LEN; i++) {
		buf[MAGIC_LEN - 1 - i] = '0' + tmp % 10;
		tmp = tmp / 10; 
	}
	p = f->f_op->write(f, buf, MAGIC_LEN, &f->f_pos);
	if (p != MAGIC_LEN) {
		return -1;
	}
	filp_close(f, NULL);
	return 0;	
}

