#ifndef SECURITYCHECK_H
#define SECURITYCHECK_H

#include "hipp.h"
#include "pstate.h"

int is_protected(struct inode* inode);
int has_integrity(struct inode* inode);
int use_security_field(struct inode* inode);
int get_read_protected_level(struct inode* inode);
int get_write_protected_level(struct inode* inode);
int get_integrity_level(struct inode* inode);
int read_check(struct dentry* dentry, struct pstate_struct* pstate);
int write_check(struct dentry* dentry, struct pstate_struct* pstate);
int create_check(struct dentry* dentry, struct pstate_struct* pstate);
int unlink_check(struct dentry* dentry, struct pstate_struct* pstate);
int create_link_check(struct dentry* dentry_from, struct dentry* dentry_to, struct pstate_struct* pstate);
int rename_check(struct dentry* dentry_from, struct dentry* dentry_to, struct pstate_struct* pstate);
int exec_check(struct dentry* dentry, struct pstate_struct* pstate);
int capability_check(int cap, struct pstate_struct* pstate);
int set_permissions_check(struct dentry* dentry, struct pstate_struct* pstate, int new_mode, int new_uid, int* sticky);
int read_from(struct dentry* dentry, struct pstate_struct* pstate);
int exec_from(struct dentry* dentry, struct pstate_struct* pstate);
int write_to(struct dentry* dentry, struct pstate_struct* pstate);
void unix_local_domain_send(struct pstate_struct* pstate, struct inode* inode);
void unix_local_domain_recv(struct pstate_struct* pstate, struct inode* inode);
void msg_queue_send(struct pstate_struct* pstate, struct msg_msg* msg);
void msg_queue_recv(struct pstate_struct* pstate, struct msg_msg* msg);
int same_uid(int a, int b);
int setuid_check(int id0, int id1, int id2, struct pstate_struct* pstate, int cap);
int shm_drop(struct shm_state_struct* shm_state);
int pstate_drop(struct pstate_struct* pstate);
void loopback_send(struct pstate_struct* pstate, struct sock* sk);
void loopback_recv(struct pstate_struct* pstate, struct sock* sk);

#endif
